package jsr.daemon;

/**
 * interface supplied by daemons in order to be hosted by a daemon container
 *
 * @author   Thomas Kopp
 * @version  0.9, 2000-11-16
 */
public interface Daemon {
    
    /**
     * initialization method to be called via a daemon control interface
     * method has to be implemented by a daemon in order to be initialized if applicable
     * @exception DaemonException thrown by an implementing daemon if initialization fails
     */
    public void init() throws DaemonException;

    /**
     * destruction method to be called via a daemon control interface
     * method has to be implemented by a daemon in order to be shutdown if applicable
     */
    public void destroy();

    /**
     * activity state indication of a daemon
     * method has to be implemented by a daemon for documenting its current activity state
     * @return true if and only if the implementing daemon is active
     */
    public boolean isActive();
    
    /**
     * descriptive information to be supplied by a daemon
     * @return description of the implementing daemon
     */
    public String getDaemonInfo();
    
    /**
     * access method for obtaining the configuration interface of a daemon
     * @return configuration interface of the implementing daemon
     */
    public DaemonConfig getDaemonConfig();
}
